if (typeof(JsTab)=="undefined") var JsTab = (function () {
    var editDialog;
    var layout;

    // -------------------- standard input utility functions
    function disableProp(prop,value) {
        var inp=document.getElementById("jsTab."+prop);
        if (inp!=null) inp.readOnly=value;
    }
    function disableCheckbox(prop,value) {
        var inp=document.getElementById("jsTab."+prop);
        if (inp!=null) inp.disabled=value;
    }
    function disableSelect(prop,value) {
        var inp=document.getElementById("jsTab."+prop);
        if (inp!=null) inp.disabled=value;
    }

    function setProp(prop,value) {
        var inp=document.getElementById("jsTab."+prop);
        if (typeof(value)=="undefined" || value=="undefined") value="";
        if (inp!=null) inp.value=value;
    }
    function setCheckbox(prop,value) {
        var inp=document.getElementById("jsTab."+prop);
        if (inp!=null) inp.checked=value;
    }
    function setSelect(prop,value) {
        var inp=document.getElementById("jsTab."+prop);
        if (inp!=null) {
            var i=inp.options.length;
            while (-1<--i) if ((""+inp.options[i].value)==value) inp.selectedIndex=i;
        }
    }
    function updateDisplay(managedLayout) {
        if (managedLayout==null) return;
        layout = managedLayout.layout;
        setProp("js",layout.js);
        setCheckbox("PieNG",layout.pieNG);
        setCheckbox("useScriptaculous",layout.useScriptaculous);
        setCheckbox("useJquery",layout.useJquery);
        setCheckbox("useLazyLoad",layout.useLazyLoad);
        setCheckbox("useLabelsJS",layout.useLabelsJS);
        var disabled=!ManageLayouts.hasLayoutPermission(ManageLayouts.EDIT_PERMISSION);
        var advanced=!(ManageLayouts.hasLayoutPermission(ManageLayouts.EDIT_PERMISSION) && ManageLayouts.hasGlobalPermission(ManageLayouts.EDIT_ADVANCED));
        disableCheckbox("js", advanced);
        disableCheckbox("PieNG", disabled);
        disableCheckbox("useScriptaculous", advanced);
        disableCheckbox("useJquery", disabled);
        disableCheckbox("useLazyLoad", disabled || !layout.useJquery);
        disableCheckbox("useLabelsJS", advanced);
    }

    function getProp(prop) {
        var inp=document.getElementById("jsTab."+prop);
        if (inp!=null) return inp.value;
        return null;
    }
    function getCheckbox(prop) {
        var inp=document.getElementById("jsTab."+prop);
        if (inp!=null) return inp.checked;
        return null;
    }
    function getSelect(prop) {
        var inp=document.getElementById("jsTab."+prop);
        if (inp!=null) return inp.options[inp.selectedIndex].value
        return null;
    }
    function getCSSprop(prop, existing) {
        var inp=document.getElementById("jsTab."+prop);
        if (inp!=null) {
            var value=getProp(prop);
            if (value==null || value.length==0) {
                value = "/* empty */";
            }
            return value;
        }
        return existing;
    }
    function updateSettings(layout) {
        layout.js=getCSSprop("js",layout.js);
        layout.pieNG=getCheckbox("PieNG");
        layout.useScriptaculous=getCheckbox("useScriptaculous");
        layout.useJquery=getCheckbox("useJquery");
        layout.useLazyLoad=getCheckbox("useLazyLoad");
        layout.useLabelsJS=getCheckbox("useLabelsJS");
        return layout;
    }

    function genTab(tab){
        return "<table class=\"layoutParameter-table\" style=\"width:98%;height:98%\">"+
                    "<tbody>"+
                        "<tr class=\"layoutParameter-row\"\">"+
                            "<td class=\"layoutParameter-property\">"+
                                "<div class=\"layoutParameter-section\">"+
                                    "<div class=\"layoutParameter-title\">"+tab+"</div>"+
                                    "<textarea class=\"layoutParameter-property\" id=\"jsTab."+tab+"\" rows=\"21\" style=\"width:100%;height:100%;\" onchange=\"ManageLayouts.changed(true);\">"+
                                        ((layout!=null) ? layout[tab] : "")+
                                    "</textarea>"+
                                "</div>"+
                            "</td>"+
                        "</tr>"+
                    "</tbody>"+
                "</table>";
    }

    function applyDialogSelection() {
        editDialog.hide();
    }

    function cancelDialogSelection() {
        updateDisplay(ManageLayouts.getManagedLayout());
        editDialog.hide();
    }

   function resizeTextBox (){
       var w=editDialog.size.width-44;
       var h=editDialog.size.height-144;
       Ext.get("jsTab.js").setSize(w,h);
   };

    function editContent(tab) {
        if (editDialog==null) {
            editDialog = new Ext.LayoutDialog("jsEdit-dlg", {
                modal:false,
                width:700,
                height:500,
                shadow:true,
                minWidth:300,
                minHeight:300,
                autoCreate:true,
                constraintoviewport:false,
                center:{
                    tabPosition:"top",
                    alwaysShowTabs:true,
                    autoCreate:true
                }});
            var editLayout = editDialog.getLayout();
            editLayout.beginUpdate();
            editLayout.add('center', new Ext.ContentPanel('jsTab-js', {title: 'Javascript', autoScroll: true, autoCreate: true}));
            editLayout.endUpdate();
            var tabPanel = editLayout.getRegion("center").getTabs();
            tabPanel.getTab("jsTab-js").setContent(genTab("js"));
            editDialog.addButton('OK', applyDialogSelection);
            editDialog.addButton('Cancel', cancelDialogSelection);
            editDialog.addListener("resize",resizeTextBox);
            updateDisplay(ManageLayouts.getManagedLayout());
        }
        editDialog.getLayout().getRegion("center").getTabs().activate("jsTab-"+tab);
        editDialog.show();
    }

    function clearValue(prop) {
        if ((ManageLayouts.hasLayoutPermission(ManageLayouts.EDIT_PERMISSION) && ManageLayouts.hasGlobalPermission(ManageLayouts.EDIT_ADVANCED))) {
            ManageLayouts.clearValue(prop);
        }
    }

//    var active=true;
//    function setActive(tabPanel, activeTab) {
//        active = (activeTab.id=="jsTab");
//        if (active) updateDisplay(ManageLayouts.getManagedLayout());
//    }
//    Ext.get("layoutmanager").borderLayout.getRegion("center").getTabs().addListener("tabchange",setActive);
    function layoutLoaded(managedLayout) {
//        if (active) updateDisplay(managedLayout);
        updateDisplay(managedLayout);
    }
    updateDisplay(ManageLayouts.getManagedLayout());

    return {updateSettings:updateSettings,
        updateDisplay:updateDisplay,
        layoutLoaded:layoutLoaded,
        clearValue:clearValue,
        editContent:editContent};
})();

