if (typeof(ManageLayouts)=="undefined") var ManageLayouts = (function () {
    // -------------------- shared AO variables
    var DEFAULT_LAYOUT_ID = "DEFAULT";
    var selectedNodeID=DEFAULT_LAYOUT_ID;
    var previousNodeId=DEFAULT_LAYOUT_ID;
    var unsavedChanges=false;
    var managedLayout;// the currently selected theme's data
    var toolbar;// the toolbar
    var children;// the raw hierarchy data (list of each layout's children keyed by layout id)
    var layoutTree;// the nlstree object
    var layoutTreeDD;// the nlstreedd object
    var borderLayout;// the borderLayout object
    var globalPermissions={};
    var defaultLayout=DEFAULT_LAYOUT_ID;

    // global permissions
    var MANAGE_LAYOUTS = "manageLayouts";
    var EDIT_ADVANCED = "editAdvanced";
    var EDIT_CSS = "editCSS";
    // layout permissions
    var VIEW_PERMISSION = "viewPermission";
    var EDIT_PERMISSION = "editPermission";
    var EDITPANEL_PERMISSION = "editPanelPermission";
    var CHILD_PERMISSION = "childPermission";


    // -------------------- Permissions
    function onGetGlobalPermissions(permissions) {
        globalPermissions=permissions;
    }
    callDWR("getGlobalPermissions",[onGetGlobalPermissions]);

    // check permission
    function hasGlobalPermission(perm) {
        return globalPermissions[perm];
    }

    function hasLayoutPermission(perm) {
        return managedLayout[perm];
    }

    // -------------------- icon control
    var iconPath="/download/resources/com.adaptavist.confluence.themes.sitebuilder:sitebuilder/icons/";
    var iconStyles={readonly:["document_lock.png","document_lock.png"],
                    layout:["document.png","document.png"],
                    unsaved:["document_warning.png","document_warning.png"],
                    newlayout:["document_add.png","document_add.png"]};
    function getIcons(style) {
        if (iconStyles[style]!=null) return contextPath+iconPath+iconStyles[style][0]+","+contextPath+iconPath+iconStyles[style][1];
        return null;
    }

    // -------------------- toolbar control utility functions
    function disableToolbtn(id,state) {
        // disables a toolbar button
        if ((toolbar!=null) && (toolbar.items!=null)) {
            var btn = toolbar.items.get(id);
            if (btn!=null) {
                btn[(state) ? "disable" : "enable"]();
            }
        }
    }
    function pressToolbtn(id,state) {
        // presses/releases a toolbar button
        if ((toolbar!=null) && (toolbar.items!=null) && (toolbar.items[id]!=null)) {
            var btn = toolbar.items.get(id);
            if (btn!=null) {
                btn.toggle(state);
            }
        }
    }
    function setToolbarState() {
        if (managedLayout==null) return;
        if (managedLayout.layout.layoutId==DEFAULT_LAYOUT_ID) {
            managedLayout[VIEW_PERMISSION]=true;
            managedLayout[EDIT_PERMISSION]=false;
            managedLayout[EDITPANEL_PERMISSION]=true;
            managedLayout[CHILD_PERMISSION]=true;
        }
        disableToolbtn("toolbar-save",!hasLayoutPermission(EDIT_PERMISSION));
        disableToolbtn("toolbar-reload",!unsavedChanges);
        disableToolbtn("toolbar-clear",!hasLayoutPermission(EDIT_PERMISSION));
        disableToolbtn("toolbar-delete",!hasLayoutPermission(EDIT_PERMISSION));
        disableToolbtn("toolbar-new",!hasLayoutPermission(CHILD_PERMISSION));
        disableToolbtn("toolbar-duplicate",!(hasLayoutPermission(CHILD_PERMISSION) && managedLayout.layout.layoutId!=DEFAULT_LAYOUT_ID));

        var noSiblings = (children[managedLayout.layout.parentId]==null || children[managedLayout.layout.parentId].length<2);
        disableToolbtn("toolbar-previous",noSiblings);
        disableToolbtn("toolbar-parent",managedLayout.layout.layoutId==DEFAULT_LAYOUT_ID);
        disableToolbtn("toolbar-next",noSiblings);
        disableToolbtn("toolbar-switch",previousNodeId==null || selectedNodeID==previousNodeId);
    }
    var lastClick=(new Date).getTime();
    function debounce() {
        var now=(new Date).getTime();
        return (now-lastClick<500);
    }

    // -------------------- tab loading
    function loadedTabContent(activeTab) {
        return function (tabcontent) {
            Visibility.removeClass(document.body,"waitCursor");
            Visibility.addClass(document.body,"normalCursor");
            activeTab.setContent(tabcontent,true);
        }
    }
    function loadTabContent(tabPanel, activeTab) {
        if (!activeTab.loaded) {
            activeTab.loaded=true;// dont really want to try and load it twice ;)
            Visibility.removeClass(document.body,"normalCursor");
            Visibility.addClass(document.body,"waitCursor");
            callDWR("getTab",[activeTab.id,loadedTabContent(activeTab)]);
        }
    }

    // page title
    var note="";
    function updateTitle(n) {
        if (n!=null) note=n;
        var t=((!hasLayoutPermission(EDIT_PERMISSION) || managedLayout.layout.layoutId==DEFAULT_LAYOUT_ID) ? " (Read Only)" : ((unsavedChanges) ? " (changed)" : "")+note);
        document.title="Adaptavist Theme Builder: Editing layout - "+managedLayout.layout.layoutId+t;
        borderLayout.getRegion("center").titleTextEl.innerHTML="Layout Editor : "+managedLayout.layout.layoutId+"<span style=\"color:red;\">"+t+"</span>";
    }

    // -------------------- layout loading
    var clickedNodeId=null;
    function layoutLoaded(data) {
        if (data==null) {// allow nodes with missing data to be selected (so that they can be removed)
            alert("Unable to locate layout with id : "+clickedNodeId)
            layoutTree.selectNodeById(clickedNodeId);
            previousNodeId=selectedNodeID;
            selectedNodeID=clickedNodeId;
            return;
        }
        unsavedChanges=false;
        Visibility.removeClass(document.body,"waitCursor");
        Visibility.addClass(document.body,"normalCursor");
        Visibility.setVisibility("panel-editor",false);
        managedLayout=data;
        updateTitle("");
        var layout = managedLayout.layout;
        layoutTree.selectNodeById(layout.layoutId);
        previousNodeId=selectedNodeID;
        selectedNodeID=layout.layoutId;
        setToolbarState();
        if (typeof(DescriptionTab)!="undefined") DescriptionTab.layoutLoaded(managedLayout);
        if (typeof(LayoutTab)!="undefined") LayoutTab.layoutLoaded(managedLayout);
        if (typeof(MenuTab)!="undefined") MenuTab.layoutLoaded(managedLayout);
        if (typeof(CssTab)!="undefined") CssTab.layoutLoaded(managedLayout);
        if (typeof(JsTab)!="undefined") JsTab.layoutLoaded(managedLayout);
        if (typeof(OptionsTab)!="undefined") OptionsTab.layoutLoaded(managedLayout);
        if (typeof(PermissionsTab)!="undefined") PermissionsTab.layoutLoaded(managedLayout);
        if (typeof(FullPreviewTab)!="undefined") FullPreviewTab.layoutLoaded(managedLayout);
        var wrapall=document.getElementById("wrapall");
        if (wrapall!=null) {
            wrapall.className = (hasLayoutPermission(EDIT_PERMISSION)) ? "" : "read-only";
        }
    }
    function checkUnsaved(callback) {
        if (unsavedChanges){
            if (confirm("You have unsaved changes, save them now?")) {
                saveLayout(true,false,callback);
                return false;
            } else {
                var node = layoutTree.getNodeById(selectedNodeID);
                if (node!=null) node.ic=getIcons("layout").split(",");
                layoutTree.reloadNode(selectedNodeID);
                unsavedChanges=false;
            }
        }
        return true;
    }
    function continueLoadLayout(state) {
        Visibility.removeClass(document.body,"normalCursor");
        Visibility.addClass(document.body,"waitCursor");
        callDWR("getLayout",[clickedNodeId,layoutLoaded]);
    }
    function loadLayout(e,treeNodeID) {
        if (debounce()) return;
        clickedNodeId=treeNodeID;
        if (checkUnsaved(continueLoadLayout)) {
            continueLoadLayout();
        }
    }

    // -------------------- layout initialisation
    function initBorderLayout() {
        if (borderLayout!=null) return;
        borderLayout = new Ext.BorderLayout("layoutmanager", {
            hideOnLayout: true,
            autoCreate: true,
            west: {
                split:true,
                initialSize: 265,
                minSize: 265,
                maxSize: 400,
                titlebar: true,
                collapsible: true,
                animate: true,
                autoCreate: true,
                autoScroll: true
            },
//            south: {
//                split:true,
//                initialSize: 100,
//                minSize: 100,
//                maxSize: 200,
//                titlebar: true,
//                collapsible: true,
//                animate: true,
//                autoCreate: true,
//                autoScroll: true
//            },
            center: {
                titlebar: true,
                title: 'Layout Editor',
                tabPosition: 'top',
                autoCreate: true,
                autoScroll: true
            }
        });
        Ext.get("layoutmanager").borderLayout=borderLayout;
        // build the layout
        borderLayout.beginUpdate();
//        borderLayout.add('south', new Ext.ContentPanel('helppanel', {title: 'Information', autoScroll: false, closable: true, autoCreate: true}));
        borderLayout.add('west', new Ext.ContentPanel('treepanel', {title: 'Available Layouts', autoScroll: true, autoCreate: true}));
        borderLayout.add('center', new Ext.ContentPanel('descriptionTab', {title: 'Overview', autoScroll: false, autoCreate: true}));
        borderLayout.add('center', new Ext.ContentPanel('layoutTab', {title: 'Layout', autoScroll: false, autoCreate: true}));
        borderLayout.add('center', new Ext.ContentPanel('menuTab', {title: 'Menus', autoScroll: false, autoCreate: true}));
        borderLayout.add('center', new Ext.ContentPanel('cssTab', {title: 'CSS', autoScroll: false, autoCreate: true}));
        borderLayout.add('center', new Ext.ContentPanel('jsTab', {title: 'JS', autoScroll: false, autoCreate: true}));
        borderLayout.add('center', new Ext.ContentPanel('optionsTab', {title: 'Options', autoScroll: false, autoCreate: true}));
        borderLayout.add('center', new Ext.ContentPanel('permissionsTab', {title: 'Permissions', autoScroll: false, autoCreate: true}));
//        borderLayout.add('center', new Ext.ContentPanel('fullpreview', {title: 'Preview', autoScroll: false, autoCreate: true}));
        borderLayout.endUpdate();
        // update the tab-panel AO
        tabPanel = borderLayout.getRegion("center").getTabs();
        tabPanel.addListener("tabchange",loadTabContent);
        tabPanel.activate('descriptionTab');
        // update the tree panel
        Ext.get("treepanel").update('<div id="extoolbar"></div><div id="treeview"></div>');
        // create the ext toolbar
        toolbar = new Ext.Toolbar("extoolbar");
        toolbar.add({cls: 'x-btn-icon',
                id:"toolbar-save",
                handler:saveLayout,
                icon: contextPath+"/download/resources/com.adaptavist.confluence.themes.sitebuilder:sitebuilder/icons/disk_blue.png",
                tooltip:"Save changes to selected layout"});
        toolbar.add("-");
        toolbar.add({cls: 'x-btn-icon',
                id:"toolbar-new",
                handler:newLayout,
                icon: contextPath+"/download/resources/com.adaptavist.confluence.themes.sitebuilder:sitebuilder/icons/document_plain_new.png",
                tooltip:"New layout based on selected layout"});
        toolbar.add({cls: 'x-btn-icon',
                id:"toolbar-duplicate",
                handler:duplicateLayout,
                icon: contextPath+"/download/resources/com.adaptavist.confluence.themes.sitebuilder:sitebuilder/icons/documents_new.png",
                tooltip:"Duplicate selected layout"});
        toolbar.add("-");
        toolbar.add({cls: 'x-btn-icon',
                id:"toolbar-reload",
                handler:reloadLayout,
                icon: contextPath+"/download/resources/com.adaptavist.confluence.themes.sitebuilder:sitebuilder/icons/document_refresh.png",
                tooltip:"Reload current layout"});
        toolbar.add({cls: 'x-btn-icon',
                id:"toolbar-clear",
                handler:clearLayout,
                icon: contextPath+"/download/resources/com.adaptavist.confluence.themes.sitebuilder:sitebuilder/icons/document_plain.png",
                tooltip:"Revert layout to defaults"});
        toolbar.add({cls: 'x-btn-icon',
                id:"toolbar-delete",
                handler:deleteLayout,
                icon: contextPath+"/download/resources/com.adaptavist.confluence.themes.sitebuilder:sitebuilder/icons/document_delete.png",
                tooltip:"Delete the selected layout"});
        toolbar.add("-");
        toolbar.add({cls: 'x-btn-icon',
                id:"toolbar-switch",
                handler:switchLayouts,
                icon: contextPath+"/download/resources/com.adaptavist.confluence.themes.sitebuilder:sitebuilder/icons/exchange.png",
                tooltip:"Switch to last selected layout"});
        toolbar.add({cls: 'x-btn-icon',
                id:"toolbar-parent",
                handler:parentLayout,
                icon: contextPath+"/download/resources/com.adaptavist.confluence.themes.sitebuilder:sitebuilder/icons/arrow_left_green.png",
                tooltip:"Switch to the parent layout"});
        toolbar.add({cls: 'x-btn-icon',
                id:"toolbar-previous",
                handler:prevLayout,
                icon: contextPath+"/download/resources/com.adaptavist.confluence.themes.sitebuilder:sitebuilder/icons/arrow_up_green.png",
                tooltip:"Switch to the previous layout"});
        toolbar.add({cls: 'x-btn-icon',
                id:"toolbar-next",
                handler:nextLayout,
                icon: contextPath+"/download/resources/com.adaptavist.confluence.themes.sitebuilder:sitebuilder/icons/arrow_down_green.png",
                tooltip:"Switch to the next layout"});
        // initialise the toolbar & treeview
        Ext.QuickTips.init();
        borderLayout.layout();// force update
        setTimeout(initTree,333);
    }


    // -------------------- tree initialisation
    function createBranch (layoutId, data) {
      var branch={};
      if (data[layoutId]!=null) {
        var o = data[layoutId].reverse();
        var i = o.length;
        while (-1<--i) {
          branch[o[i]] = createBranch(o[i],data);
        }
      }
      return branch;
    }
    function generateChildNodes(parentNodeId,childNodeData){
        var i;
        for (i in childNodeData) {
            layoutTree.add(i,parentNodeId,i,"",getIcons("layout"),true,false,{id:i,parent:parentNodeId},"description");
            layoutTree.setDrag(i,true,false);
            layoutTree.setDrop(i,true,false);
            if (children[i]==null) children[i]=[];
            generateChildNodes(i,childNodeData[i]);
        }
    }
    var onNodeDrop;
    function treeInit(data) {
        children=data;
        Visibility.removeClass(document.body,"waitCursor");
        Visibility.addClass(document.body,"normalCursor");
        layoutTree=new NlsTree("layoutTree");
        layoutTreeDD = new NlsTreeDD("layoutTree");
        onNodeDrop=layoutTreeDD.onNodeDrop;
        layoutTree.treeOnClick=loadLayout;
        layoutTreeDD.onNodeDrop=reparentLayout;
        layoutTree.add(DEFAULT_LAYOUT_ID, 0, DEFAULT_LAYOUT_ID, "", getIcons("readonly"), true, false, {id:"DEFAULT",parent:"DEFAULT"}, "Default layout (read only)");
        layoutTree.setDrag(DEFAULT_LAYOUT_ID,false,false);
        layoutTree.setDrop(DEFAULT_LAYOUT_ID,true,false);
        generateChildNodes(DEFAULT_LAYOUT_ID,createBranch(DEFAULT_LAYOUT_ID,data));
        layoutTree.renderAttributes("treeviewDD");
        layoutTree.render("treeview");
        loadLayout(null,defaultLayout);
    }
    var tree_timeout = 0;
    function initTree() {
        if (typeof(dwr.engine)!="undefined" && typeof(ManageLayoutsDWR)!="undefined" && typeof(NlsTree)!="undefined" && typeof(NlsTreeDD)!="undefined") {
            Visibility.removeClass(document.body,"normalCursor");
            Visibility.addClass(document.body,"waitCursor");
            callDWR("getHierarchy",[treeInit]);
        } else {
            if (tree_timeout++ < 120) {
                setTimeout(initTree, 333);
            } else {
                alert("manage-layouts.js : failed tree initialisation");
            }
        }
    }


    // -------------------- save layout
    function onSaveLayout(callback) {
        return function (state) {
            if (state!=null) alert(state);
            if (typeof(callback)=="function") callback(state);
            updateTitle("");
        }
    }
    function recordChanges() {
        if (managedLayout==null) return;
        var layout = managedLayout.layout;
        if (typeof(DescriptionTab)!="undefined") DescriptionTab.updateSettings(layout);
        if (typeof(LayoutTab)!="undefined") LayoutTab.updateSettings(layout);
        if (typeof(MenuTab)!="undefined") MenuTab.updateSettings(layout);
        if (typeof(CssTab)!="undefined") CssTab.updateSettings(layout);
        if (typeof(JsTab)!="undefined") JsTab.updateSettings(layout);
        if (typeof(OptionsTab)!="undefined") OptionsTab.updateSettings(layout);
        if (typeof(PermissionsTab)!="undefined") PermissionsTab.updateSettings(layout);
    }
    function saveLayout(dontConfirm,dontRecordChanges,callback) {
        if (debounce()) return;
        if (!hasLayoutPermission(EDIT_PERMISSION)) return alert("You do not have permission to edit this layout.");
        if ((typeof(dontConfim) == "Boolean" && !dontConfirm) && !confirm("Save changes?")) return;
        if (typeof(dontRecordChanges) != "Boolean" || !dontRecordChanges) recordChanges();
        var layout = managedLayout.layout;
        unsavedChanges=false;
        var node = layoutTree.getNodeById(layout.layoutId);
        node.ic=getIcons("layout").split(",");
        layoutTree.reloadNode(layout.layoutId);
        callDWR("setLayout",[layout,onSaveLayout(callback)]);
        setToolbarState();
    }

    // -------------------- reload layout
    function reloadLayout() {
        if (debounce()) return;
        unsavedChanges=false;
        if (confirm("Reload and remove all unsaved changes from this layout?")) {
            var node = layoutTree.getNodeById(managedLayout.layout.layoutId);
            node.ic=getIcons("layout").split(",");
            layoutTree.reloadNode(managedLayout.layout.layoutId);
            loadLayout(null,selectedNodeID);
        }
    }

    // -------------------- clear layout
    function onClearLayout(data) {
        data.layout.layoutId = managedLayout.layout.layoutId;
        data.layout.parentId = managedLayout.layout.parentId;
        data.layout.description = managedLayout.layout.description;
        layoutLoaded(data);
        changed();
    }
    function clearLayout() {
        if (debounce()) return;
        if (!hasLayoutPermission(EDIT_PERMISSION)) return alert("You do not have permission to edit this layout");
        if ((managedLayout.layout.parentId!=null) && confirm("Remove all customsations from this layout?")) {
            callDWR("getLayout",[managedLayout.layout.parentId,onClearLayout]);
        }
    }

    // -------------------- clear a parameter
    function onClearValue(error) {
        if (error) alert(error);
    }
    function clearValue(parameter) {
        if (debounce()) return;
        if (confirm("Clear the '"+parameter+"' parameter?")) {
            checkUnsaved();
            callDWR("clearValue",[managedLayout.layout.layoutId, parameter, onClearValue])
            reloadLayout();
        }
    }

    // -------------------- delete layout
    function onDeleteLayout(status) {
        if (status) return alert(status);
        var node = layoutTree.getNodeById(selectedNodeID);
        var parent = node.xtra.parent;
        layoutTree.remove(selectedNodeID);
        delete children[selectedNodeID];
        previousNodeId=selectedNodeID=null;
        unsavedChanges=false;
        loadLayout(null,parent);
    }
    function deleteLayout() {
        if (debounce()) return;
        var node = layoutTree.getNodeById(selectedNodeID);
        if (node.fc!=null) return alert("You can't delete this layout because other layouts are based on it. Delete or move the child layouts first.");
        if (!hasLayoutPermission(EDIT_PERMISSION)) return alert("You do not have permission to delete this layout");
        if (confirm("Delete layout with Id : "+selectedNodeID+" ?")) {
            callDWR("deleteLayout",[selectedNodeID,onDeleteLayout]);
        }
    }

    // -------------------- reparent (drag/drop) layout
    function onReparentLayout(status) {
        if (status) alert(status);
        reloadLayout();
    }
    function reparentLayout(e) {
        checkUnsaved();
        var droptarget=nlsddSession.destData.xtra.id;
        if (droptarget!=null) {
            var dragged=nlsddSession.srcData;
            var i=dragged.length;
            while (-1<--i) {
                if (dragged[i].xtra.id!=null) callDWR("reparentLayout",[dragged[i].xtra.id, droptarget,onReparentLayout])
            }
        }
        onNodeDrop(e);
    }

    // -------------------- utility function to request layoutId input from user
    function requestLayoutId() {
        var layoutID=prompt("Enter Layout ID");
        if (layoutID!=null) {
            if (layoutID.length<1) {
                alert("Please enter a layout ID");
                return requestLayoutId();
            }
            layoutID=layoutID.toUpperCase();
            if (!layoutID.match(/^[A-Z0-9]+$/)) {
                alert("Invalid layout ID : "+layoutID);
                return requestLayoutId();
            }
            if (children[layoutID]!=null) {
                alert("Layout ID '"+layoutID+"' already in use, please select another!");
                return requestLayoutId();
            }
        }
        return layoutID;
    }

    // -------------------- new layout
    function newLayout() {
        if (debounce()) return;
        checkUnsaved();
        if (managedLayout==null || typeof(managedLayout.layout)=="undefined") return alert("Please select a parent layout");// this should NEVER happen (tm)
        if (!hasLayoutPermission(CHILD_PERMISSION)) return alert("You do not have permission to create layouts based on the currently selected layout");
        tabPanel.activate('descriptionTab');
        var layoutId=requestLayoutId();
        if (layoutId==null) return;
        recordChanges();
        var layout = managedLayout.layout;
        layout.parentId=layout.layoutId;
        layout.layoutId=layoutId;
        layout.name="Child of "+layout.parentId;
        layout.description=layout.name+"\n"+layout.description;
        layout.importCSS="/* empty */";
        layout.customCSS="/* empty */";
        layout.ieCSS="/* empty */";
        previousNodeId=selectedNodeID;
        selectedNodeID=layoutId;
        layoutTree.add(layoutId,layout.parentId,layoutId,"",getIcons("newlayout"),true,false,{id:layoutId, parent:layout.parentId},"description");
        layoutTree.reloadNode(layout.parentId);
        layoutTree.expandNode(layout.parentId);
        layoutTree.selectNodeById(layoutId);
        managedLayout[VIEW_PERMISSION]=true;
        managedLayout[EDIT_PERMISSION]=true;
//        managedLayout[EDITPANEL_PERMISSION]=true;
        managedLayout[CHILD_PERMISSION]=true;
        layoutLoaded(managedLayout);
        saveLayout(true,true);
        if (children[layoutId]==null) children[layoutId]=[];
        layoutLoaded(managedLayout);
    }

    // -------------------- duplicate layout
    function duplicateLayout() {
        if (debounce()) return;
        checkUnsaved();
        if (managedLayout==null || typeof(managedLayout.layout)=="undefined") return alert("Please select a layout to duplicate");// this should NEVER happen (tm)
        if (!hasLayoutPermission(CHILD_PERMISSION)) return alert("You do not have permission to create layouts based on the currently selected layout");
        tabPanel.activate('descriptionTab');
        var layoutId=requestLayoutId();
        if (layoutId==null) return;
        recordChanges();
        var layout = managedLayout.layout;
        layout.name="Duplicate of "+layout.layoutId;
        layout.description=layout.name+"\n"+layout.description;
        layout.layoutId=layoutId;
        previousNodeId=selectedNodeID;
        selectedNodeID=layoutId;
        layoutTree.add(layoutId,layout.parentId,layoutId,"",getIcons("newlayout"),true,false,{id:layoutId, parent:layout.parentId},"description");
        layoutTree.reloadNode(layout.parentId);
        layoutTree.expandNode(layout.parentId);
        layoutTree.selectNodeById(layoutId);
        managedLayout[VIEW_PERMISSION]=true;
        managedLayout[EDIT_PERMISSION]=true;
//        managedLayout[EDITPANEL_PERMISSION]=true;
        managedLayout[CHILD_PERMISSION]=true;
        layoutLoaded(managedLayout);
        saveLayout(true,true);
        if (children[layoutId]==null) children[layoutId]=[];
        layoutLoaded(managedLayout);
    }

    // -------------------- utility function to mark the node as having changed
    function changed(record) {
        unsavedChanges=true;
        var node = layoutTree.getNodeById(selectedNodeID);
        node.ic=getIcons("unsaved").split(",");
        layoutTree.reloadNode(selectedNodeID);
        setToolbarState();
        updateTitle();
        if (record) {
            recordChanges();
        }
    }

    // -------------------- go to previous layout
    function prevLayout() {
        if (debounce()) return;
        var siblings=children[managedLayout.layout.parentId];
        var i=siblings.length;
        while (-1<--i) if (siblings[i]==managedLayout.layout.layoutId) break;
        if (++i>=siblings.length) i=0;
        loadLayout(null,siblings[i]);
    }

    // -------------------- go to parent layout
    function parentLayout() {
        if (debounce()) return;
        if (managedLayout.layout.parentId!=null) loadLayout(null,managedLayout.layout.parentId);
    }

    // -------------------- go to next layout
    function nextLayout() {
        if (debounce()) return;
        var siblings=children[managedLayout.layout.parentId];
        var i=siblings.length;
        while (-1<--i) if (siblings[i]==managedLayout.layout.layoutId) break;
        if (--i<0) i=siblings.length-1;
        loadLayout(null,siblings[i]);
    }

    // -------------------- switch to the last used layout
    function switchLayouts() {
        loadLayout(null,previousNodeId);
    }

    function setDefaultLayout(dl) {
        defaultLayout=(typeof(dl)=="undefined" || dl==null || dl.length==0) ? DEFAULT_LAYOUT_ID : dl;
        Ext.EventManager.onDocumentReady(initBorderLayout);
    }

    // -------------------- callDWR
    function setWaiting(state) {
        Ext.get("layoutmanager").setOpacity((state) ? 0.3 : 1,false);
    }

    function callDWR(method,args) {
        var callBack = args[args.length-1];
        args[args.length-1] = function(){setWaiting(false);callBack.apply(this,arguments);};
        setWaiting(true);
        return ManageLayoutsDWR[method].apply(ManageLayoutsDWR,args);
    }


    // -------------------- return 'public' api
    return {saveLayout:saveLayout,
        reloadLayout:reloadLayout,
        clearLayout:clearLayout,
        clearValue:clearValue,
        deleteLayout:deleteLayout,
        newLayout:newLayout,
        duplicateLayout:duplicateLayout,
        updateTitle:updateTitle,
        hasGlobalPermission:hasGlobalPermission,
        hasLayoutPermission:hasLayoutPermission,
        MANAGE_LAYOUTS:MANAGE_LAYOUTS,
        EDIT_ADVANCED:EDIT_ADVANCED,
        EDIT_CSS:EDIT_CSS,
        VIEW_PERMISSION:VIEW_PERMISSION,
        EDIT_PERMISSION:EDIT_PERMISSION,
        EDITPANEL_PERMISSION:EDITPANEL_PERMISSION,
        CHILD_PERMISSION:CHILD_PERMISSION,
        changed:changed,
        getManagedLayout:function(){return managedLayout;},
        prevLayout:prevLayout,
        parentLayout:parentLayout,
        nextLayout:nextLayout,
        switchLayouts:switchLayouts,
        setDefaultLayout:setDefaultLayout,
        callDWR:callDWR};
})();

