if (typeof(Palette)=="undefined") var Palette = (function () {
    var CELLCHARS="&nbsp;&nbsp;&nbsp;";
    var SQUARE_SIZE=11;
    var NUM_COLS=29;
    var namedColours = ["#C0C0C0","#808080","#800000","#008000","#000080","#800080","#808000","#008080","#FFFFFF","#000000","#FF0000","#00FF00","#0000FF","#FF00FF","#FFFF00","#00FFFF"];
    var NUM_ROWS=namedColours.length;
    var target;

    var recentColours = [];

    function RGB2HSB(RGBstr){
        var red = parseInt(RGBstr.substr(1,2),16);
        var gre = parseInt(RGBstr.substr(3,2),16);
        var blu = parseInt(RGBstr.substr(5,2),16);
        if ((red==NaN) || (gre==NaN) || (blu==NaN)) return -1;
        var max = Math.max(red,Math.max(gre,blu));
        var min = Math.min(red,Math.min(gre,blu));
        var colHSB = {H:0,S:0,B:Math.round(max*100/255)};
        if(max!=min){
            colHSB.S = Math.round(100*(max-min)/max);
            var hue;
            var tmpR = (max-red)/(max-min);
            var tmpG = (max-gre)/(max-min);
            var tmpB = (max-blu)/(max-min);
            if(red==max){
                hue = tmpB - tmpG;
            }else if(gre==max){
                hue = 2 + tmpR - tmpB;
            }else if(blu==max){
                hue = 4 + tmpG - tmpR;
            }
            colHSB.H = (Math.round(hue*60) + 360)%360;
        }
        return colHSB;
    }
    function XX(s) {
        while (s.length<2) s="0"+s;
        return s;
    }
    function HSB2RGB(hue,sat,bri){
        var red, green, blue;
        sat=sat%101;
        bri=bri%101;
        if(sat==0){
            red = bri;
            green = bri;
            blue = bri;
        }else{
            hue = (hue+360)%360;
            var hue2 = Math.floor(hue/60);
            var dif = (hue/60)-hue2;
            sat/=100;
            bri/=100;
            var mid1 = bri*(1-sat*dif);
            var mid2 = bri*(1-sat*(1-dif));
            var min = bri*(1-sat);
            if(hue2 == 0){
                red = bri;
                green= mid2;
                blue = min;
            } else if(hue2 == 1) {
                red = mid1;
                green = bri;
                blue = min;
            } else if(hue2 == 2) {
                red = min;
                green = bri;
                blue = mid2;
            } else if(hue2 == 3) {
                red = min;
                green = mid1;
                blue = bri;
            } else if(hue2 == 4) {
                red = mid2;
                green = min;
                blue = bri;
            } else {
                red = bri;
                green = min;
                blue = mid1;
            }
        }
        red = (red==NaN) ? 0 : Math.min(Math.round(red*255),255);
        green = (green==NaN) ? 0 : Math.min(Math.round(green*255),255);
        blue = (blue==NaN) ? 0 : Math.min(Math.round(blue*255),255);
        return "#"+XX(red.toString(16))+XX(green.toString(16))+XX(blue.toString(16));
    }

    function generateCell(colour,title,style,id,call) {
        return "<td id='"+id+"' style='background:"+colour+";width:"+SQUARE_SIZE+"px;height:"+SQUARE_SIZE+"px;'><div style='width:"+SQUARE_SIZE+"px;height:"+SQUARE_SIZE+"px;overflow:hidden;cursor:pointer;"+style+"' onclick='"+call+"' title='"+title+"'>"+CELLCHARS+"</div></td>";
    }

    function limit(x, l) {
        return Math.min(l,(x<0) ? 0 : x);
    }


    /*
    var LTB1="border-left:1px solid #5A6B7D;border-bottom:1px solid #5A6B7D;border-top:1px solid #5A6B7D;overflow:hidden;";
    var TB1="border-bottom:1px solid #5A6B7D;border-top:1px solid #5A6B7D;overflow:hidden;";
    var RTB1="border-right:1px solid #5A6B7D;border-bottom:1px solid #5A6B7D;border-top:1px solid #5A6B7D;overflow:hidden;"

    function modulate(colour,shortSpan) {
        html = generateCell(colour,"Colour : "+colour,LTB1,"S"+colour,"Palette.setColour(\""+colour+"\",true);");// #1
        var orig=RGB2HSB(colour);
        var comp={H:orig.H,S:orig.S};
        var comp2={};
        var comp3={};
        if (orig.B > 70) {
          comp.B=orig.B-30;
            colour=HSB2RGB(orig.H,orig.S,orig.B-15);
            html += generateCell(colour,"Colour : "+colour,TB1,"S"+colour,"Palette.setColour(\""+colour+"\",true);");// #2
        } else {
          comp.B=orig.B+30;
            colour=HSB2RGB(orig.H,orig.S,orig.B+15);
            html += generateCell(colour,"Colour : "+colour,TB1,"S"+colour,"Palette.setColour(\""+colour+"\",true);");// #2
        }
        colour=HSB2RGB(comp.H,comp.S,comp.B);
        html += generateCell(colour,"Colour : "+colour,((shortSpan==true) ? RTB1 : TB1),"S"+colour,"Palette.setColour(\""+colour+"\",true);");// #3
        if (!shortSpan) {
            if (orig.H<30) {
                comp  = {H:orig.H+20, S:orig.S, B:orig.B};
                comp2 = {H:orig.H+20, S:orig.S};
                comp3 = {H:orig.H+20, S:orig.S};
                if (orig.B > 70) {
                    comp2.B=orig.B-30;
                    comp3.B=orig.B-15;
                } else {
                    comp2.B=orig.B+30;
                    comp3.B=orig.B+15;
                }
            } else if (orig.H<60) {
                comp  = {H:orig.H+150, S:limit(orig.S-30,100), B:limit(orig.B-20,100)};
                comp2 = {H:orig.H+150, S:limit(orig.S-70,100), B:limit(orig.B+20,100)};
                comp3 = {H:orig.H+150, S:limit(orig.S-70,100), B:orig.B};
            } else if (orig.H<180) {
                comp  = {H:orig.H-40, S:orig.S, B:orig.B};
                comp2 = {H:orig.H-40, S:orig.S};
                comp3 = {H:orig.H-40, S:orig.S};
                if (orig.B > 70) {
                    comp2.B=orig.B-30;
                    comp3.B=orig.B-15;
                } else {
                    comp2.B=orig.B+30;
                    comp3.B=orig.B+15;
                }
            } else if (orig.H<220) {
                comp  = {H:orig.H-160, S:orig.S, B:orig.B};
                comp2 = {H:orig.H-170, S:orig.S};
                comp3 = {H:orig.H-170, S:orig.S};
                if (orig.B > 70) {
                    comp2.B=orig.B-30;
                    comp3.B=orig.B-15;
                } else {
                    comp2.B=orig.B+30;
                    comp3.B=orig.B+15;
                }
            } else if (orig.H<300) {
                comp  = {H:orig.H, S:limit(orig.S - 60 ,100), B:orig.B};
                comp2 = {H:orig.H, S:limit(orig.S - 60 ,100)};
                comp3 = {H:orig.H, S:limit(orig.S - 60 ,100)};
                if (orig.B > 70) {
                    comp2.B=orig.B-30;
                    comp3.B=orig.B-15;
                } else {
                    comp2.B=orig.B+30;
                    comp3.B=orig.B+15;
                }
            } else {
                comp  = {H:orig.H+20, B:orig.B};
                comp2 = {H:orig.H+20};
                comp3 = {H:orig.H+20};
                if (orig.S > 50) {
                    comp3.S=comp2.S=comp.S=orig.S-40;
                } else {
                    comp3.S=comp2.S=comp.S=orig.S+40;
                }
                if (orig.B > 70) {
                    comp2.B=orig.B-30;
                    comp3.B=orig.B-15;
                } else {
                    comp2.B=orig.B+30;
                    comp3.B=orig.B+15;
                }
            }
            colour=HSB2RGB(comp.H,comp.S,comp.B);
            html += generateCell(colour,"Colour : "+colour,TB1,"S"+colour,"Palette.setColour(\""+colour+"\",true);");// #4
            colour=HSB2RGB(comp3.H,comp3.S,comp3.B);
            html += generateCell(colour,"Colour : "+colour,TB1,"S"+colour,"Palette.setColour(\""+colour+"\",true);");// #5
            colour=HSB2RGB(comp2.H,comp2.S,comp2.B);
            html += generateCell(colour,"Colour : "+colour,RTB1,"S"+colour,"Palette.setColour(\""+colour+"\",true);");// #6
    //	html += generateSample(HSB2RGB(0,0,limit(100-orig.B,100)));
    //	html += generateSample(HSB2RGB(0,0,orig.B));
    //	if (orig.B >=50) {
    //		html += generateSample(HSB2RGB(0,0,0));
    //	} else {
    //		html += generateSample(HSB2RGB(0,0,100));
    //	}
        }
        return html;
    }

    function generateSample(colour) {
        return "&nbsp;<span onclick='Palette.setColour(\""+colour+"\",true);' title='Colour : "+colour+"' style='cursor:pointer;border:2px solid "+colour+";background:"+colour+"'>"+CELLCHARS+"</span>";
    }
    function generatePreview(colour) {
        var html="<tr><td style='vertical-align:left;padding: 3px 1px 1px 2px;' align='center' colspan='"+(NUM_COLS+2)+"'><span><input type='text' id='pallete_hexvalue' class='pallete_hexvalue' size='9' onkeydown='Palette.checkKey(event);'></span>";
        html+="&nbsp;<span onclick='Palette.setColour(\""+colour+"\",true);' title='Colour : "+colour+"' style='cursor:pointer;border:4px solid "+colour+";background:"+colour+"'>"+CELLCHARS+"</span>";
        var orig = RGB2HSB(colour);
        html+=generateSample(HSB2RGB(orig.H, 70, 40));
        html+=generateSample(HSB2RGB(orig.H, 60, 80));
        html+=generateSample(HSB2RGB(orig.H, 40, 60));
        html+=generateSample(HSB2RGB(orig.H, 40, 80));
        html+=generateSample(HSB2RGB(orig.H, 30, 100));
        var contrast=180 + orig.H;
        html+=generateSample(HSB2RGB(contrast, orig.S,orig.B));
        html+=generateSample(HSB2RGB(contrast, 70, 40));
        html+=generateSample(HSB2RGB(contrast, 60, 60));
        html+=generateSample(HSB2RGB(contrast, 40, 60));
        html+=generateSample(HSB2RGB(contrast, 40, 80));
        html+=generateSample(HSB2RGB(contrast, 30, 100));
        html+="</td></tr><tr>";
        html+=modulate(colour);
        html+=modulate(HSB2RGB(25  + orig.H, orig.S,orig.B));
        html+=modulate(HSB2RGB(345 + orig.H, orig.S,orig.B));
        html+=modulate(HSB2RGB(180 + orig.H, orig.S,orig.B));
        html+=modulate(HSB2RGB(155 + orig.H, orig.S,orig.B));
        html+=modulate(HSB2RGB(205 + orig.H, orig.S,orig.B));
//        html+=modulate(HSB2RGB(90  + orig.H, orig.S,orig.B),true);
//        html+=modulate(HSB2RGB(270 + orig.H, orig.S,orig.B),true);
        html+="</tr>";
        return html;
    }
/**/

//    function inRange(value,target,range) {
//        return ((value>=target) && (value<(target+range)));
//    }
    var MIDDLE;
    var PREVIEW_STR;
    var hueMultiplier=360/NUM_COLS;
    var satMultiplier=100/NUM_ROWS;
    function updateGrid(b) {
        if (document.getElementById(target)==null) {
            var myDiv = document.createElement("div");
            target=myDiv.id="pallete";
            myDiv.style.display='none';
            myDiv.innerHTML="<div>&nbsp;</div>";
            document.body.appendChild(myDiv);
        }
        var picker = document.getElementById(target);
        var html="<table class='pallete_table' style='width:"+NUM_COLS*SQUARE_SIZE+"px;height:"+NUM_ROWS*SQUARE_SIZE+"px' cellspacing='0' cellpadding='0'>";
//        if (PREVIEW_STR==null) PREVIEW_STR=generatePreview("#CCCCCC");
//        html += PREVIEW_STR;
        if (b!=null) {
//            var targetHSB;
//            var targetColour=null;
//            if (picker.current!=null) {
//                targetHSB=RGB2HSB(picker.current);
//            }
            MIDDLE=""
            var colour;
            var h;
            var c;
            var bri;
            var s=NUM_ROWS;
            while (-1<--s) {
                MIDDLE+="<tr>";
                c=Math.floor(s*(255/NUM_ROWS));
                if (c>255) c=255;
                colour="#"+XX((255-c).toString(16))+XX((255-c).toString(16))+XX((255-c).toString(16));
                MIDDLE+=generateCell(colour,"Grey : "+(255-c),"","G"+colour.substr(-6),"Palette.setColour(\""+colour+"\");");
                colour=namedColours[s];
                if (colour==null) colour="#AAAAAA";
                MIDDLE+=generateCell(colour,"Colour : "+colour,"","N"+colour.substr(-6),"Palette.setColour(\""+colour+"\");");
                h=NUM_COLS;
                while (-1<--h) {
                    colour = HSB2RGB(Math.round(h*hueMultiplier),(1+s)*satMultiplier,b);
//                    if ((targetHSB!=undefined) && inRange(Math.round(h*hueMultiplier),targetHSB.H,hueMultiplier) && inRange(Math.round((1+s)*satMultiplier),targetHSB.S,satMultiplier)) targetColour=colour;
//                    if ((targetHSB!=undefined) && (Math.round(h*hueMultiplier)==targetHSB.H) && (Math.round((1+s)*satMultiplier)==targetHSB.S)) targetColour=colour;
                    MIDDLE+=generateCell(colour,"Colour : "+colour,"","C"+colour.substr(-6),"Palette.setColour(\""+colour+"\");");
                }
                colour="#"+XX(c.toString(16))+XX(c.toString(16))+XX(c.toString(16));
                bri=Math.floor(11*c/25);
                if (bri>100) bri=100;
                MIDDLE+=generateCell(colour,"Brightness : "+bri,((b==bri) ? "border:1px solid #3366FF;" : "")+"overflow:hidden;","B"+bri,"Palette.updateGrid("+bri+");");
                MIDDLE+="</tr>";
            }
        }
        html+=MIDDLE;
/*
        s=c=NUM_COLS+3;
        --s;
        html+="<tr>";
        while (-1<--c) {
            colour = (recentColours[s-c]!=null) ? recentColours[s-c] : "#A0A0A0";
            html+=generateCell(colour,"Colour : "+colour,"border-top:1px solid #5A6B7D;overflow:hidden;","R"+colour,"Palette.setColour(\""+colour+"\");");
        }
        html+="</tr>";
/**/
        html += "</table>";
        picker.innerHTML=html;
        picker.bri=b;
//        if ((targetColour!=null) && (b!=null)) {
//            setColour(targetColour);
//        }
    }

/*
    function checkKey(e) {
        var key=null;
        if (window.event) {
            key = window.event.keyCode;
        } else if (e) {
            key = e.which;
        }
        if (key==13) {
            var hexvalue=document.getElementById("pallete_hexvalue").value;
            var i;
            while ((i=hexvalue.indexOf("#"))>-1) hexvalue = hexvalue.substr(0,i) + hexvalue.substr(i+1);
            if ((parseInt(hexvalue,16)<1) || (hexvalue.length>6)) {
                alert("invalid value");
            } else {
                setColour("#"+hexvalue);
            }
        }
    }
/**/
/*
    function updateHistory(colour) {
        var i=recentColours.length;
        while (-1<--i) if (recentColours[i]==colour) recentColours.splice(i,1);
        recentColours.push(colour);
        while (recentColours.length>NUM_COLS+3) recentColours.splice(0,1);
    }
/**/
    function setStyle(colour,className) {
        var cell=document.getElementById("G"+colour.substr(-6));// greys
        if (cell!=null) cell.className=className;
        cell=document.getElementById("N"+colour.substr(-6));// named
        if (cell!=null) cell.className=className;
        cell=document.getElementById("C"+colour.substr(-6));// colour
        if (cell!=null) cell.className=className;
        cell=document.getElementById("R"+colour.substr(-6));// recent
        if (cell!=null) cell.className=className;
    }

    function selectColour(colour,source) {
        if (colour==null) return;
//        updateHistory(colour);
//        if (!preventPreview) PREVIEW_STR=generatePreview(colour);
        var hsb=RGB2HSB(colour);
        updateGrid(hsb.B);
        setStyle(colour,"hilightcell");
//        var hexvalue=document.getElementById("pallete_hexvalue");
//        hexvalue.value=colour.toUpperCase();
    }
    var currentcolour;
    function setColour(colour, preventPreview) {
        if (colour==currentcolour) return;
        currentcolour=colour;
        selectColour(colour, preventPreview);
        if (typeof(ColourDialog)!="undefined") {
            ColourDialog.selectColour(colour,"Palette");
        }
    }

    function showPicker(targetId) {
        target = targetId;
        selectColour(ColourDialog.getSelectedColourValue());
    }
    return {setColour:setColour,
            selectColour:selectColour,
            updateGrid:updateGrid,
//            checkKey:checkKey,
            showPicker:showPicker};
})();