if (typeof(Validation) == "undefined") var Validation = (function () {
    var validation = {};
    // checks that the value is a number
    validation.isNumeric = function (f) {
      return function (o) {
        if (Number(o.value).toString()=="NaN") {
          o.diagnostic+="FAILURE: value is not a number.";
          o.success=false;
        } else {
          o.diagnostic+="Value is a number. ";
          o.success=true;
          if (f!=null) o=f(o);
        }
        return o;
      };
    };

    // checks that the value is an integer (assumes isNumeric used before this check)
    validation.isInteger = function (f) {
      return function (o) {
        if (Number(o.value)!=Math.round(Number(o.value))) {
          o.diagnostic+="FAILURE: value is not an integer.";
          o.success=false;
        } else {
          o.diagnostic+="Value is an integer. ";
          o.success=true;
          if (f!=null) o=f(o);
        }
        return o;
      };
    };

    // checks to see that the value exists within the array passed
    validation.isVocabulary = function (a,f) {
      return function (o) {
        if (a.lastIndexOf(o.value.toLowerCase())<0) {
          o.diagnostic+="FAILURE: value is not a member of ["+a+"].";
          o.success=false;
        } else {
          o.diagnostic+="Value is a member of ["+a+"]. ";
          o.success=true;
          if (f!=null) o=f(o);
        }
        return o;
      };
    };

    // checks to see that the value appears to be a comma separated list, and that each element in the list is contained by the array (l)
    // ********************************************************************************************
    // >>>>>>>>>>>>>>>>>>>>>>>>>>> DOES NOT YET CHECK VALIDATION ARRAY <<<<<<<<<<<<<<<<<<<<<<<<<<<<
    // ********************************************************************************************
    validation.isCSL = function (a,f) {
      return function (o) {
        if (o.value.split(",").length==1) {
          o.diagnostic+="FAILURE: value is not a comma separated list.";
          o.success=false;
        } else {
          o.diagnostic+="Value is a comma separated list. ";
          o.success=true;
          if (f!=null) o=f(o);
        }
        return o;
      };
    };

    // checks that the value list within a range (assumes isNumeric used before this check)
    validation.inRange = function (Vmin,Vmax,f) {
      return function (o) {
        if ((Number(o.value)<Vmin) || (Number(o.value)>Vmax)) {
          o.diagnostic+="FAILURE: value does not lie within the range ["+Vmin+"] to ["+Vmax+"].";
          o.success=false;
        } else {
          o.diagnostic+="Value lies within the range ["+Vmin+"] to ["+Vmax+"]. ";
          o.success=true;
          if (f!=null) o=f(o);
        }
        return o;
      };
    };

    // checks that the value is less than number specified
    validation.lessThan = function (val,f) {
      return function (o) {
        if (Number(o.value)>val) {
          o.diagnostic+="FAILURE: value is greater than ["+val+"].";
          o.success=false;
        } else {
          o.diagnostic+="Value is less than ["+val+"]. ";
          o.success=true;
          if (f!=null) o=f(o);
        }
        return o;
      };
    };

    // checks that the value is greater than number specified
    validation.greaterThan = function (val,f) {
      return function (o) {
        if (Number(o.value)<val) {
          o.diagnostic+="FAILURE: value is less than ["+val+"].";
          o.success=false;
        } else {
          o.diagnostic+="Value is greater than ["+val+"]. ";
          o.success=true;
          if (f!=null) o=f(o);
        }
        return o;
      };
    };

    // Checks that the value's length does not exceed the limit supplied
    validation.maxLength = function (len,f) {
      return function (o) {
        if (o.value.length>len) {
          o.diagnostic+="FAILURE: value contains more than ["+len+"] characters.";
          o.success=false;
        } else {
          o.diagnostic+="Value contains less than (or equal to) ["+len+"] characters. ";
          o.success=true;
          if (f!=null) o=f(o);
        }
        return o;
      };
    };

    // Checks that the value's length exceeds the limit supplied
    validation.minLength = function (len,f) {
      return function (o) {
        if (o.value.length<len) {
          o.diagnostic+="FAILURE: value contains less than ["+len+"] characters.";
          o.success=false;
        } else {
          o.diagnostic+="Value contains more than (or equal to) ["+len+"] characters. ";
          o.success=true;
          if (f!=null) o=f(o);
        }
        return o;
      };
    };

    // checks that the value contains the specified character
    validation.containsChar = function (c,f) {
      return function (o) {
        if (o.value.indexOf(c)<0) {
          o.diagnostic+="FAILURE: value does not contain a ["+c+"] character.";
          o.success=false;
        } else {
          o.diagnostic+="Value contains a ["+c+"] character. ";
          o.success=true;
          if (f!=null) o=f(o);
        }
        return o;
      };
    };

    return validation;
})();

